package org.phosphoresce.webcore.ext.hibernate4.transaction;

import java.util.List;

import org.hibernate.SQLQuery;
import org.hibernate.criterion.Order;
import org.phosphoresce.lib.commons.util.AnnotationUtil;
import org.phosphoresce.webcore.ext.hibernate4.annotation.HibernateDAO;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateTransactionException;

/**
 * Hibernateによるデータベースアクセス処理クラスの共通的なインタフェースを提供します。<br>
 * <p>
 * すべてのHibernateデータベースアクセスオブジェクトクラスはこのクラスを継承して実装します。<br>
 * また、データベースアクセスオブジェクトクラスは必ずクラスアノテーションに{@link HibernateDAO}を付与し、対象のエンティティを決定する必要があります。<br>
 * {@link HibernateDAO#entity()}で指定するエンティティクラスは必ず、このクラスジェネリクスに指定するエンティティクラスに一致させる必要があります。<br>
 * </p>
 *
 * @param <E> データベースアクセス対象エンティティクラス
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 * 2013/07/21	Kitagawa		対象エンティティクラス取得をフックメソッドからアノテーションに変更
 * 2013/07/31	Kitagawa		汎用インタフェースを追加({@link #findAll(boolean)}、{@link #countAll(boolean)})
 * 2013/08/06	Kitagawa		総称型で指定したエンティティクラス以外のCriteriaを生成する汎用インタフェースを追加
 * 2013/08/13	Kitagawa		クラス総称型指定のない汎用クラス{@link AbstractHibernateDAO}と分離
 *-->
 */
public abstract class AbstractGenericHibernateDAO<E extends AbstractHibernateEntity> extends AbstractHibernateDAO {

	/**
	 * コンストラクタ<br>
	 * @param transaction トランザクション管理オブジェクト
	 */
	protected AbstractGenericHibernateDAO(HibernateTransactionManager transaction) {
		super(transaction);
	}

	/**
	 * DAOクラスで対象とされているエンティティクラスを取得します。<br>
	 * @return エンティティクラス
	 */
	@SuppressWarnings("unchecked")
	private Class<E> getEntityClass() {
		HibernateDAO hibernateDAO = AnnotationUtil.getClassAnnotation(HibernateDAO.class, getClass());
		if (hibernateDAO == null) {
			throw new HibernateTransactionException("FHBN00025", new Object[] { getClass().getName() });
		}
		return (Class<E>) hibernateDAO.entity();
	}

	/**
	 * データベースクエリ条件オブジェクトを生成します。<br>
	 * @return データベースクエリ条件オブジェクト
	 */
	protected final GenericCriteria<E> createCriteria() {
		return super.createCriteria(getEntityClass());
	}

	/**
	 * データベースクエリ条件オブジェクトを生成します。<br>
	 * 対象エンティティが削除状態を保持するエンティティの場合、削除状態のエンティティを除外する条件を持ちます。<br>
	 * @return データベースクエリ条件オブジェクト
	 */
	protected final GenericCriteria<E> createCriteriaExcludeDeleted() {
		return super.createCriteriaExcludeDeleted(getEntityClass());
	}

	/**
	 * SQLデータベースクエリを生成します。<br>
	 * DAOクラスにおいて当インタフェースを利用した場合、総称型で指定されたエンティティクラスで制限がかからないクエリとなることに注意して下さい。<br>
	 * これは利用者がDAOクラスの位置づけ(エンティティ対応しているDAO)に注意して利用する必要があり、場合によってはアプリケーション開発規約で利用禁止とすることも検討して下さい。<br>
	 * @param sql SQLデータベースクエリ
	 * @return SQLデータベースクエリオブジェクト
	 */
	protected final SQLQuery createSQLQuery(String sql) {
		return super.createSQLQuery(sql);
	}

	/**
	 * HQLデータベースクエリを生成します。<br>
	 * DAOクラスにおいて当インタフェースを利用した場合、総称型で指定されたエンティティクラスで制限がかからないクエリとなることに注意して下さい。<br>
	 * これは利用者がDAOクラスの位置づけ(エンティティ対応しているDAO)に注意して利用する必要があり、場合によってはアプリケーション開発規約で利用禁止とすることも検討して下さい。<br>
	 * @param hql HQLデータベースクエリ
	 * @return HQLデータベースクエリオブジェクト
	 */
	protected final GenericHQLQuery<E> createHQLQuery(String hql) {
		return super.createHQLQuery(getEntityClass(), hql);
	}

	/**
	 * 指定されたレコードIDでエンティティを検索します。<br>
	 * 当インタフェースは削除済の条件は加味されません。<br>
	 * @param id レコードID
	 * @return エンティティオブジェクト
	 */
	public final E findById(Long id) {
		return super.findById(getEntityClass(), id);
	}

	/**
	 * データベースに登録されている全てのエンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param includeDeleted 論理削除済みデータを含める場合にtrueを指定
	 * @param orders ソート条件オブジェクト
	 * @return エンティティレコードリスト
	 */
	public final List<E> findAll(boolean includeDeleted, List<Order> orders) {
		return super.findAll(getEntityClass(), includeDeleted, orders);
	}

	/**
	 * データベースに登録されている全ての未削除エンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param orders ソート条件オブジェクト
	 * @return エンティティレコードリスト
	 */
	public final List<E> findAll(List<Order> orders) {
		return super.findAll(getEntityClass(), false, orders);
	}

	/**
	 * データベースに登録されている全てのエンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param includeDeleted 論理削除済みデータを含める場合にtrueを指定
	 * @return エンティティレコードリスト
	 */
	public final List<E> findAll(boolean includeDeleted) {
		return super.findAll(getEntityClass(), includeDeleted, null);
	}

	/**
	 * データベースに登録されている全ての未削除エンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @return エンティティレコードリスト
	 */
	public final List<E> findAll() {
		return super.findAll(getEntityClass(), false);
	}

	/**
	 * データベースに登録されているエンティティレコード数を取得します。<br>
	 * @param includeDeleted 論理削除済みデータを含める場合にtrueを指定
	 * @return エンティティレコード数
	 */
	public final long countAll(boolean includeDeleted) {
		return super.countAll(getEntityClass(), includeDeleted);
	}

	/**
	 * データベースに登録されている未削除のエンティティレコード数を取得します。<br>
	 * @return エンティティレコード数
	 */
	public final long countAll() {
		return super.countAll(getEntityClass(), false);
	}
}
