package org.phosphoresce.webcore.ext.hibernate4.transaction;

import java.lang.reflect.Constructor;

import org.hibernate.PessimisticLockException;
import org.hibernate.StaleObjectStateException;
import org.hibernate.dialect.lock.LockingStrategyException;
import org.hibernate.exception.ConstraintViolationException;
import org.phosphoresce.webcore.core.logger.CodeConvertLogger;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateEntityDuplicateException;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateEntityLockException;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateEntityModificationException;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateLogicException;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateTransactionException;
import org.phosphoresce.webcore.ext.hibernate4.model.SUser;
import org.slf4j.LoggerFactory;

/**
 * アドオンモジュールが提供するHibernateトランザクションを利用した業務処理クラスを設ける際のインタフェースを提供します。<br>
 * <p>
 * このクラスはHibernateトランザクションアドオンモジュールを利用する際の業務処理を行う際のクラス構造を提供します。<br>
 * 但し、このクラスを直接継承してアプリケーション実装を行う設計とすることは、Hibernateに特化したアプリケーション設計となることに注意してください。<br>
 * そのため、通常はここで提供される業務処理APIと類似の汎用的なAPIを持つロジッククラス、すなわちHibernateに特化しない、たとえばJDBCトランザクションでも利用可能な、
 * 標準化されたトランザクションロジッククラスによってインタフェースが提供されることが推奨されています。<br>
 * </p>
 * <p>
 * 尚、このクラスを継承して実装されるロジッククラスは基本的に、このクラスが提供されるコンストラクタと同様のコンストラクタが提供されるクラス設計とする必要があることに注意してください。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 *-->
 */
public abstract class AbstractHibernateLogic<U extends SUser> {

	/** ロガーオブジェクト */
	protected final CodeConvertLogger log = new CodeConvertLogger(LoggerFactory.getLogger(getClass()));

	/** 処理コンテナオブジェクト */
	private HibernateTransactionContext<U> context;

	/**
	 * コンストラクタ<br>
	 * @param transactionContext 処理コンテナオブジェクト
	 */
	public AbstractHibernateLogic(HibernateTransactionContext<U> transactionContext) {
		super();
		this.context = transactionContext;
	}

	/**
	 * ビジネスロジック処理を実行します。<br>
	 */
	public final void execute() {
		try {
			HibernateTransactionManager transaction = context.getTransaction();
			transaction.open();

			process();

			transaction.commit();
			transaction.close();
		} catch (LockingStrategyException e) {
			log.output(e, "FHBN00033");
			throw new HibernateEntityLockException(e);
		} catch (PessimisticLockException e) {
			log.output(e, "FHBN00033");
			throw new HibernateEntityLockException(e);
		} catch (StaleObjectStateException e) {
			log.output(e, "FHBN00033");
			throw new HibernateEntityModificationException(e);
		} catch (ConstraintViolationException e) {
			log.output(e, "FHBN00033");
			throw new HibernateEntityDuplicateException(e);
		} catch (Throwable e) {
			throw new HibernateLogicException("FHBN00022", e);
		}
	}

	/**
	 * ビジネスロジック処理を実行します。<br>
	 * @throws  Throwable ビジネスロジック処理中に予期せぬ例外が発生した場合にスローされます
	 */
	protected abstract void process() throws Throwable;

	/**
	 * 指定されたサービスクラスのクラスインスタンスを生成します。<br>
	 * @param clazz サービスクラス
	 * @return サービスクラスインスタンス
	 */
	protected <S extends AbstractHibernateService<U>> S createService(Class<S> clazz) {
		try {
			Constructor<S> constructor = clazz.getConstructor(new Class[] { HibernateTransactionContext.class });
			constructor.setAccessible(false);
			S instance = constructor.newInstance(new Object[] { context });
			return instance;
		} catch (Throwable e) {
			throw new HibernateTransactionException("FHBN00011", e);
		}
	}

	/**
	 * 指定されたロジッククラスのクラスインスタンスを生成します。<br>
	 * 自身のロジッククラスから別のロジッククラスを利用する際に利用しますが、クラス間の結合度が高くなるため、
	 * 相互利用を行う際のクラス設計が煩雑にならないように注意してください。<br>
	 * @param clazz ロジッククラス
	 * @return ロジッククラスインスタンス
	 */
	protected <G extends AbstractHibernateLogic<SUser>> G createLogic(Class<G> clazz) {
		try {
			Constructor<G> constructor = clazz.getConstructor(new Class[] { HibernateTransactionContext.class });
			constructor.setAccessible(false);
			G instance = constructor.newInstance(new Object[] { context });
			return instance;
		} catch (Throwable e) {
			throw new HibernateTransactionException("FHBN00024", e);
		}
	}
}
