package org.phosphoresce.webcore.ext.hibernate4.transaction;

import java.util.List;

import org.phosphoresce.webcore.ext.hibernate4.model.SUser;

/**
 * エンティティに依存しない汎用的なデータベースサービスインタフェースを提供します。<br>
 * <p>
 * このクラスではエンティティの登録、変更、削除等、エンティティに依存しない共通のサービスインタフェースを提供します。<br>
 * 各アプリケーションごとに作成するサブロジッククラスにおいて、共通フィールドとして設けることで、エンティティ毎の冗長的な処理が削減出来ます。<br>
 * </p>
 * <p>
 * 但し、このクラスはサブサービスクラスで本来集約された処理を行う為に設けられている共通処理をpublicなインタフェースとするため、
 * 実装時の規約において制限を設けておかない場合、ロジッククラスとのスコープが曖昧になります。<br>
 * その為、初期クラス設計としては、上位ロジッククラス{@link AbstractHibernateLogic}では、共通フィールドとして提供されていません。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/08/16	Kitagawa		新規作成
 *-->
 */
public class GeneralHibernateService<U extends SUser> extends AbstractHibernateService<U> {

	/** 汎用データベースアクセスDAO */
	private GeneralHibernateDAO generalDAO;

	/**
	 * コンストラクタ<br>
	 * @param transactionContext トランザクション処理コンテキストオブジェクト
	 */
	public GeneralHibernateService(HibernateTransactionContext<U> transactionContext) {
		super(transactionContext);
		generalDAO = createDAO(GeneralHibernateDAO.class);
	}

	/**
	 * エンティティをレコードIDで検索します。<br>
	 * @param entityClass 対象エンティティクラス
	 * @param id レコードID
	 * @return エンティティオブジェクト
	 */
	public final <E extends AbstractHibernateEntity> E findById(Class<E> entityClass, Long id) {
		return generalDAO.findById(entityClass, id);
	}

	/**
	 * エンティティを保存します。<br>
	 * @param entity エンティティオブジェクト
	 */
	public final void saveEntity(AbstractHibernateEntity entity) {
		if (entity.getId() != null) {
			lockPessimisticNoWait(entity);
		}
		super.save(entity);
	}

	/**
	 * エンティティを保存します。<br>
	 * @param entities エンティティオブジェクト
	 */
	public final <E extends AbstractHibernateEntity> void saveEntity(List<E> entities) {
		for (E entity : entities) {
			if (entity.getId() != null) {
				lockPessimisticNoWait(entity);
			}
		}
		super.save(entities);
	}

	/**
	 * エンティティを論理削除します。<br>
	 * @param entity エンティティオブジェクト
	 */
	public final void deleteEntity(AbstractHibernateEntity entity) {
		lockPessimisticNoWait(entity);
		super.delete(entity);
	}

	/**
	 * エンティティを論理削除します。<br>
	 * @param entities エンティティオブジェクト
	 */
	public final <E extends AbstractHibernateEntity> void deleteEntity(List<E> entities) {
		for (E entity : entities) {
			lockPessimisticNoWait(entity);
		}
		super.delete(entities);
	}

	/**
	 * エンティティを物理削除します。<br>
	 * @param entity エンティティオブジェクト
	 */
	public final void removeEntity(AbstractHibernateEntity entity) {
		lockPessimisticNoWait(entity);
		super.remove(entity);
	}

	/**
	 * エンティティを論理削除します。<br>
	 * @param entities エンティティオブジェクト
	 */
	public final <E extends AbstractHibernateEntity> void removeEntity(List<E> entities) {
		for (E entity : entities) {
			lockPessimisticNoWait(entity);
		}
		super.remove(entities);
	}
}
