package org.phosphoresce.webcore.ext.hibernate4.transaction;

import java.util.List;

import org.hibernate.CacheMode;
import org.hibernate.Criteria;
import org.hibernate.FetchMode;
import org.hibernate.FlushMode;
import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.ScrollMode;
import org.hibernate.ScrollableResults;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projection;
import org.hibernate.criterion.Projections;
import org.hibernate.sql.JoinType;
import org.hibernate.transform.ResultTransformer;
import org.phosphoresce.webcore.core.config.Config;
import org.phosphoresce.webcore.ext.hibernate4.HibernateConfigName;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateTooManyResultException;

/**
 * Hibernateフレームワークのセッションが提供する{@link Criteria}インスタンスをラップするクラスでインタフェースは{@link Criteria}を提供します。<br>
 * <p>
 * 当クラスは扱うエンティティを制限するクラスジェネリクス(count文等によるLong型返却等も発生する為、ジェネリクスはObject型としています)が設けられて提供されますが、
 * インタフェースは全てHibernateの{@link Session}が提供した{@link Criteria}に委譲するため、このクラスのAPIは全て{@link Criteria}に準拠します。<br>
 * アドオンフレームワーク上で利用するCriteriaクラスはすべてこのクラスが利用されます。<br>
 * </p>
 * <p>
 * 尚、当クラスが提供されるメソッドにおいてDeplicate属性が付与されたメソッドは、Hibernate側の非推奨以外にフレームワーク側で意図的に制限している場合があります。<br>
 * そのため、このクラスでは非推奨のインタフェース利用制限するために、該当のインタフェース利用時に{@link NoSuchMethodError}がスローするようにクラス設計されています。<br>
 * これは、明示的に利用制限を行うためのクラス設計であり、ユーザーはこの制限に準じたクラス利用設計を行う必要があります。<br>
 * </p>
 * <p>
 * Hibernate JavaDoc (4.1.12.Final)<br>
 * <a href="http://docs.jboss.org/hibernate/orm/4.1/javadocs/">http://docs.jboss.org/hibernate/orm/4.1/javadocs/</a>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 *-->
 */
//public class GenericCriteria<E extends AbstractHibernateEntity> implements Criteria {
public class GenericCriteria<E extends Object> implements Criteria {

	/** HibernateCriteriaオブジェクト */
	private Criteria criteria;

	/**
	 * コンストラクタ<br>
	 * @param criteria HibernateCriteriaオブジェクト
	 */
	public GenericCriteria(Criteria criteria) {
		if (criteria == null) {
			throw new NullPointerException("criteria");
		}
		this.criteria = criteria;
	}

	/**
	 * 条件に合致するレコード件数を取得します。<br>
	 * @return レコード件数
	 */
	public long count() {
		long count = (Long) criteria.setProjection(Projections.rowCount()).uniqueResult();
		criteria.setProjection(null);
		return count;
	}

	/**
	 * Get the alias of the entity encapsulated by this criteria instance.<br>
	 * @return The alias for the encapsulated entity.
	 * @see org.hibernate.Criteria#getAlias()
	 */
	public String getAlias() {
		return criteria.getAlias();
	}

	/**
	 * Used to specify that the query results will be a projection (scalar in nature). Implicitly specifies the CriteriaSpecification.PROJECTION result transformer.<br>
	 * The individual components contained within the given projection determines the overall "shape" of the query result.<br>
	 * @param projection The projection representing the overall "shape" of the query results.
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setProjection(org.hibernate.criterion.Projection)
	 */
	public GenericCriteria<E> setProjection(Projection projection) {
		Criteria resultCriteria = criteria.setProjection(projection);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Add a restriction to constrain the results to be retrieved.<br>
	 * @param criterion The criterion object representing the restriction to be applied.
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#add(org.hibernate.criterion.Criterion)
	 */
	public GenericCriteria<E> add(Criterion criterion) {
		Criteria resultCriteria = criteria.add(criterion);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Add a restriction to constrain the results to be retrieved.<br>
	 * @param order The order object representing an ordering to be applied to the results.
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#addOrder(org.hibernate.criterion.Order)
	 */
	public GenericCriteria<E> addOrder(Order order) {
		Criteria resultCriteria = criteria.addOrder(order);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Specify an association fetching strategy for an association or a collection of values.<br>
	 * @param associationPath a dot seperated property path
	 * @param fetchmode The fetch mode for the referenced association
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setFetchMode(java.lang.String, org.hibernate.FetchMode)
	 */
	public GenericCriteria<E> setFetchMode(String associationPath, FetchMode fetchmode) throws HibernateException {
		Criteria resultCriteria = criteria.setFetchMode(associationPath, fetchmode);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Set the lock mode of the current entity<br>
	 * @param lockmode The lock mode to be applied
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setLockMode(org.hibernate.LockMode)
	 */
	public GenericCriteria<E> setLockMode(LockMode lockmode) {
		Criteria resultCriteria = criteria.setLockMode(lockmode);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Set the lock mode of the aliased entity<br>
	 * @param alias The previously assigned alias representing the entity to which the given lock mode should apply.
	 * @param lockmode The lock mode to be applied
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setLockMode(java.lang.String, org.hibernate.LockMode)
	 */
	public GenericCriteria<E> setLockMode(String alias, LockMode lockmode) {
		Criteria resultCriteria = criteria.setLockMode(alias, lockmode);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Join an association, assigning an alias to the joined association.<br>
	 * Functionally equivalent to createAlias(String, String, int) using CriteriaSpecification.INNER_JOIN for the joinType.<br>
	 * @param associationPath A dot-seperated property path
	 * @param alias The alias to assign to the joined association (for later reference).
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#createAlias(java.lang.String, java.lang.String)
	 */
	public GenericCriteria<E> createAlias(String associationPath, String alias) throws HibernateException {
		Criteria resultCriteria = criteria.createAlias(associationPath, alias);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Join an association using the specified join-type, assigning an alias to the joined association.<br>
	 * The joinType is expected to be one of CriteriaSpecification.INNER_JOIN (the default), CriteriaSpecification.FULL_JOIN, or CriteriaSpecification.LEFT_JOIN.<br>
	 * @param associationPath A dot-seperated property path
	 * @param alias The alias to assign to the joined association (for later reference).
	 * @param joinType The type of join to use.
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#createAlias(java.lang.String, java.lang.String, org.hibernate.sql.JoinType)
	 */
	public GenericCriteria<E> createAlias(String associationPath, String alias, JoinType joinType) throws HibernateException {
		Criteria resultCriteria = criteria.createAlias(associationPath, alias, joinType);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Join an association using the specified join-type, assigning an alias to the joined association.<br>
	 * The joinType is expected to be one of CriteriaSpecification.INNER_JOIN (the default), CriteriaSpecification.FULL_JOIN, or CriteriaSpecification.LEFT_JOIN.<br>
	 * @param associationPath A dot-seperated property path
	 * @param alias The alias to assign to the joined association (for later reference).
	 * @param joinType The type of join to use.
	 * @return this (for method chaining)
	 * @deprecated use {@link #createAlias(String, String, org.hibernate.sql.JoinType)}
	 * @see org.hibernate.Criteria#createAlias(java.lang.String, java.lang.String, int)
	 */
	@Override
	@Deprecated
	public Criteria createAlias(String associationPath, String alias, int joinType) throws HibernateException {
		throw new NoSuchMethodError("deplicated");
		//Criteria resultCriteria = criteria.createAlias(associationPath, alias, joinType);
		//if (criteria.equals(resultCriteria)) {
		//	return this;
		//} else {
		//	return new GenericCriteria<E>(resultCriteria);
		//}
	}

	/**
	 * Join an association using the specified join-type, assigning an alias to the joined association.<br>
	 * The joinType is expected to be one of CriteriaSpecification.INNER_JOIN (the default), CriteriaSpecification.FULL_JOIN, or CriteriaSpecification.LEFT_JOIN.<br>
	 * @param associationPath A dot-seperated property path
	 * @param alias The alias to assign to the joined association (for later reference).
	 * @param joinType The type of join to use.
	 * @param withClause The criteria to be added to the join condition (ON clause)
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#createAlias(java.lang.String, java.lang.String, org.hibernate.sql.JoinType, org.hibernate.criterion.Criterion)
	 */
	public GenericCriteria<E> createAlias(String associationPath, String alias, JoinType joinType, Criterion withClause) throws HibernateException {
		Criteria resultCriteria = criteria.createAlias(associationPath, alias, joinType, withClause);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Join an association using the specified join-type, assigning an alias to the joined association.<br>
	 * The joinType is expected to be one of CriteriaSpecification.INNER_JOIN (the default), CriteriaSpecification.FULL_JOIN, or CriteriaSpecification.LEFT_JOIN.<br>
	 * @param associationPath A dot-seperated property path
	 * @param alias The alias to assign to the joined association (for later reference).
	 * @param joinType The type of join to use.
	 * @param withClause The criteria to be added to the join condition (ON clause)
	 * @return this (for method chaining)
	 * @deprecated use {@link #createAlias(String, String, JoinType, Criterion)}
	 * @see org.hibernate.Criteria#createAlias(java.lang.String, java.lang.String, int, org.hibernate.criterion.Criterion)
	 */
	@Override
	@Deprecated
	public Criteria createAlias(String associationPath, String alias, int joinType, Criterion withClause) throws HibernateException {
		throw new NoSuchMethodError("deplicated");
		//Criteria resultCriteria = criteria.createAlias(associationPath, alias, joinType, withClause);
		//if (criteria.equals(resultCriteria)) {
		//	return this;
		//} else {
		//	return new GenericCriteria<E>(resultCriteria);
		//}
	}

	/**
	 * Create a new Criteria, "rooted" at the associated entity.<br>
	 * Functionally equivalent to createCriteria(String, int) using CriteriaSpecification.INNER_JOIN for the joinType.<br>
	 * @param associationPath A dot-seperated property path
	 * @return the created "sub criteria"
	 * @see org.hibernate.Criteria#createCriteria(java.lang.String)
	 */
	public GenericCriteria<E> createCriteria(String associationPath) throws HibernateException {
		Criteria resultCriteria = criteria.createCriteria(associationPath);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Create a new Criteria, "rooted" at the associated entity, using the specified join type.<br>
	 * @param associationPath A dot-seperated property path
	 * @param joinType The type of join to use.
	 * @return the created "sub criteria"
	 * @see org.hibernate.Criteria#createCriteria(java.lang.String, org.hibernate.sql.JoinType)
	 */
	public GenericCriteria<E> createCriteria(String associationPath, JoinType joinType) throws HibernateException {
		Criteria resultCriteria = criteria.createCriteria(associationPath, joinType);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Create a new Criteria, "rooted" at the associated entity, using the specified join type.<br>
	 * @param associationPath A dot-seperated property path
	 * @param joinType The type of join to use.
	 * @return the created "sub criteria"
	 * @deprecated use {@link #createAlias(String, String, org.hibernate.sql.JoinType)}
	 * @see org.hibernate.Criteria#createCriteria(java.lang.String, int)
	 */
	@Override
	@Deprecated
	public Criteria createCriteria(String associationPath, int joinType) throws HibernateException {
		throw new NoSuchMethodError("deplicated");
		//Criteria resultCriteria = criteria.createCriteria(associationPath, joinType);
		//if (criteria.equals(resultCriteria)) {
		//	return this;
		//} else {
		//	return new GenericCriteria<E>(resultCriteria);
		//}
	}

	/**
	 * Create a new Criteria, "rooted" at the associated entity, assigning the given alias.<br>
	 * Functionally equivalent to createCriteria(String, String, int) using CriteriaSpecification.INNER_JOIN for the joinType.<br>
	 * @param associationPath A dot-seperated property path
	 * @param alias The alias to assign to the joined association (for later reference).
	 * @return the created "sub criteria"
	 * @see org.hibernate.Criteria#createCriteria(java.lang.String, java.lang.String)
	 */
	public GenericCriteria<E> createCriteria(String associationPath, String alias) throws HibernateException {
		Criteria resultCriteria = criteria.createCriteria(associationPath, alias);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Create a new Criteria, "rooted" at the associated entity, assigning the given alias and using the specified join type.<br>
	 * @param associationPath A dot-seperated property path
	 * @param alias The alias to assign to the joined association (for later reference).
	 * @param joinType The type of join to use.
	 * @return the created "sub criteria"
	 * @see org.hibernate.Criteria#createCriteria(java.lang.String, java.lang.String, org.hibernate.sql.JoinType)
	 */
	public GenericCriteria<E> createCriteria(String associationPath, String alias, JoinType joinType) throws HibernateException {
		Criteria resultCriteria = criteria.createCriteria(associationPath, alias, joinType);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Create a new Criteria, "rooted" at the associated entity, assigning the given alias and using the specified join type.<br>
	 * @param associationPath A dot-seperated property path
	 * @param alias The alias to assign to the joined association (for later reference).
	 * @param joinType The type of join to use.
	 * @return the created "sub criteria"
	 * @deprecated use {@link #createCriteria(String, org.hibernate.sql.JoinType)}
	 * @see org.hibernate.Criteria#createCriteria(java.lang.String, java.lang.String, int)
	 */
	@Override
	@Deprecated
	public Criteria createCriteria(String associationPath, String alias, int joinType) throws HibernateException {
		throw new NoSuchMethodError("deplicated");
		//Criteria resultCriteria = criteria.createCriteria(associationPath, alias, joinType);
		//if (criteria.equals(resultCriteria)) {
		//	return this;
		//} else {
		//	return new GenericCriteria<E>(resultCriteria);
		//}
	}

	/**
	 * Create a new Criteria, "rooted" at the associated entity, assigning the given alias and using the specified join type.<br>
	 * @param associationPath A dot-seperated property path
	 * @param alias The alias to assign to the joined association (for later reference).
	 * @param joinType The type of join to use.
	 * @param withClause The criteria to be added to the join condition (ON clause)
	 * @return the created "sub criteria"
	 * @see org.hibernate.Criteria#createCriteria(java.lang.String, java.lang.String, org.hibernate.sql.JoinType, org.hibernate.criterion.Criterion)
	 */
	public GenericCriteria<E> createCriteria(String associationPath, String alias, JoinType joinType, Criterion withClause) throws HibernateException {
		Criteria resultCriteria = criteria.createCriteria(associationPath, alias, joinType, withClause);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Create a new Criteria, "rooted" at the associated entity, assigning the given alias and using the specified join type.<br>
	 * @param associationPath - A dot-seperated property path
	 * @param alias The alias to assign to the joined association (for later reference).
	 * @param joinType The type of join to use.
	 * @param withClause The criteria to be added to the join condition (ON clause)
	 * @return the created "sub criteria"
	 * @deprecated use {@link #createCriteria(String, String, org.hibernate.sql.JoinType, org.hibernate.criterion.Criterion)}
	 * @see org.hibernate.Criteria#createCriteria(java.lang.String, java.lang.String, int, org.hibernate.criterion.Criterion)
	 */
	@Override
	@Deprecated
	public Criteria createCriteria(String associationPath, String alias, int joinType, Criterion withClause) throws HibernateException {
		throw new NoSuchMethodError("deplicated");
		//Criteria resultCriteria = criteria.createCriteria(associationPath, alias, joinType, withClause);
		//if (criteria.equals(resultCriteria)) {
		//	return this;
		//} else {
		//	return new GenericCriteria<E>(resultCriteria);
		//}
	}

	/**
	 * Set a strategy for handling the query results. This determines the "shape" of the query result.<br>
	 * @param resultTransformer The transformer to apply
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setResultTransformer(org.hibernate.transform.ResultTransformer)
	 */
	public GenericCriteria<E> setResultTransformer(ResultTransformer resultTransformer) {
		Criteria resultCriteria = criteria.setResultTransformer(resultTransformer);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Set a limit upon the number of objects to be retrieved.<br>
	 * @param maxResults the maximum number of results
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setMaxResults(int)
	 */
	public GenericCriteria<E> setMaxResults(int maxResults) {
		Criteria resultCriteria = criteria.setMaxResults(maxResults);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Set the first result to be retrieved.<br>
	 * @param firstResult the first result to retrieve, numbered from 0
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setFirstResult(int)
	 */
	public GenericCriteria<E> setFirstResult(int firstResult) {
		Criteria resultCriteria = criteria.setFirstResult(firstResult);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Was the read-only/modifiable mode explicitly initialized?<br>
	 * @return true, the read-only/modifiable mode was explicitly initialized; false, otherwise.
	 * @see org.hibernate.Criteria#isReadOnlyInitialized()
	 */
	public boolean isReadOnlyInitialized() {
		return criteria.isReadOnlyInitialized();
	}

	/**
	 * Should entities and proxies loaded by this Criteria be put in read-only mode? If the read-only/modifiable setting was not initialized, then the default read-only/modifiable setting for the persistence context is returned instead.
	 * @return true, entities and proxies loaded by the criteria will be put in read-only mode false, entities and proxies loaded by the criteria will be put in modifiable mode
	 * @see org.hibernate.Criteria#isReadOnly()
	 */
	public boolean isReadOnly() {
		return criteria.isReadOnly();
	}

	/**
	 * Set the read-only/modifiable mode for entities and proxies loaded by this Criteria. This setting overrides the default setting for the persistence context.
	 * @param readOnly true, entities and proxies loaded by the criteria will be put in read-only mode false, entities and proxies loaded by the criteria will be put in modifiable mode
	 * @see org.hibernate.Criteria#setReadOnly(boolean)
	 */
	public GenericCriteria<E> setReadOnly(boolean readOnly) {
		Criteria resultCriteria = criteria.setReadOnly(readOnly);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Set a fetch size for the underlying JDBC query.
	 * @param fetchSize the fetch size
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setFetchSize(int)
	 */
	public GenericCriteria<E> setFetchSize(int fetchSize) {
		Criteria resultCriteria = criteria.setFetchSize(fetchSize);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Set a timeout for the underlying JDBC query.
	 * @param timeout The timeout value to apply.
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setTimeout(int)
	 */
	public GenericCriteria<E> setTimeout(int timeout) {
		Criteria resultCriteria = criteria.setTimeout(timeout);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Enable caching of this query result, provided query caching is enabled for the underlying session factory.
	 * @param cacheable Should the result be considered cacheable; default is to not cache (false).
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setCacheable(boolean)
	 */
	public GenericCriteria<E> setCacheable(boolean cacheable) {
		Criteria resultCriteria = criteria.setCacheable(cacheable);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Set the name of the cache region to use for query result caching.
	 * @param cacheRegion the name of a query cache region, or null for the default query cache
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setCacheRegion(java.lang.String)
	 */
	public GenericCriteria<E> setCacheRegion(String cacheRegion) {
		Criteria resultCriteria = criteria.setCacheRegion(cacheRegion);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Add a comment to the generated SQL.
	 * @param comment a human-readable string
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setComment(java.lang.String)
	 */
	public GenericCriteria<E> setComment(String comment) {
		Criteria resultCriteria = criteria.setComment(comment);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Override the flush mode for this particular query.
	 * @param flushMode The flush mode to use.
	 * @return this (for method chaining)
	 * @see org.hibernate.Criteria#setFlushMode(org.hibernate.FlushMode)
	 */
	public GenericCriteria<E> setFlushMode(FlushMode flushMode) {
		Criteria resultCriteria = criteria.setFlushMode(flushMode);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Override the cache mode for this particular query.
	 * @param cacheMode The cache mode to use.
	 * @see org.hibernate.Criteria#setCacheMode(org.hibernate.CacheMode)
	 */
	public GenericCriteria<E> setCacheMode(CacheMode cacheMode) {
		Criteria resultCriteria = criteria.setCacheMode(cacheMode);
		if (criteria.equals(resultCriteria)) {
			return this;
		} else {
			return new GenericCriteria<E>(resultCriteria);
		}
	}

	/**
	 * Get the results.
	 * @return The list of matched query results.
	 * @see org.hibernate.Criteria#list()
	 */
	@SuppressWarnings("unchecked")
	public List<E> list() throws HibernateException {
		List<E> list = null;
		if (Config.getInteger(HibernateConfigName.HIBERNATE_RESULT_MAXCOUNT) >= 1) {
			criteria.setMaxResults(Config.getInteger(HibernateConfigName.HIBERNATE_RESULT_MAXCOUNT) + 1);
			list = criteria.list();
			if (list.size() > Config.getLong(HibernateConfigName.HIBERNATE_RESULT_MAXCOUNT)) {
				throw new HibernateTooManyResultException();
			}
		}
		return (List<E>) criteria.list();
	}

	/**
	 * Get the results as an instance of ScrollableResults
	 * @return The ScrollableResults representing the matched query results.
	 * @see org.hibernate.Criteria#scroll()
	 */
	public ScrollableResults scroll() throws HibernateException {
		return criteria.scroll();
	}

	/**
	 * Get the results as an instance of ScrollableResults based on the given scroll mode.
	 * @param scrollMode Indicates the type of underlying database cursor to request.
	 * @return The ScrollableResults representing the matched query results.
	 * @see org.hibernate.Criteria#scroll(org.hibernate.ScrollMode)
	 */
	public ScrollableResults scroll(ScrollMode scrollMode) throws HibernateException {
		return criteria.scroll(scrollMode);
	}

	/**
	 * Convenience method to return a single instance that matches the query, or null if the query returns no results.
	 * @return the single result or null
	 * @see org.hibernate.Criteria#uniqueResult()
	 */
	@SuppressWarnings("unchecked")
	public E uniqueResult() throws HibernateException {
		return (E) criteria.uniqueResult();
	}
}
