package org.phosphoresce.webcore.ext.hibernate4.transaction;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.hibernate.CacheMode;
import org.hibernate.FlushMode;
import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.LockOptions;
import org.hibernate.Query;
import org.hibernate.ScrollMode;
import org.hibernate.ScrollableResults;
import org.hibernate.transform.ResultTransformer;
import org.hibernate.type.Type;
import org.phosphoresce.webcore.core.config.Config;
import org.phosphoresce.webcore.ext.hibernate4.HibernateConfigName;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateTooManyResultException;

/**
 * Hibernateフレームワークのセッションが提供する{@link Query}インスタンスをラップするクラスでインタフェースは{@link Query}を提供します。<br>
 * <p>
 * 当クラスは扱うエンティティを制限するクラスジェネリクス(count文等によるLong型返却等も発生する為、ジェネリクスはObject型としています)が設けられて提供されますが、
 * インタフェースは全てHibernateの{@link Session}が提供した{@link Query}に委譲するため、このクラスのAPIは全て{@link Query}に準拠します。<br>
 * アドオンフレームワーク上で利用するQueryクラスはすべてこのクラスが利用されます。<br>
 * </p>
 * <p>
 * 尚、当クラスが提供されるメソッドにおいてDeplicate属性が付与されたメソッドは、Hibernate側の非推奨以外にフレームワーク側で意図的に制限している場合があります。<br>
 * そのため、このクラスでは非推奨のインタフェース利用制限するために、該当のインタフェース利用時に{@link NoSuchMethodError}がスローするようにクラス設計されています。<br>
 * これは、明示的に利用制限を行うためのクラス設計であり、ユーザーはこの制限に準じたクラス利用設計を行う必要があります。<br>
 * </p>
 * <p>
 * Hibernate JavaDoc (4.1.12.Final)<br>
 * <a href="http://docs.jboss.org/hibernate/orm/4.1/javadocs/">http://docs.jboss.org/hibernate/orm/4.1/javadocs/</a>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 *-->
 */
//public class GenericHQLQuery<E extends AbstractHibernateEntity> implements Query {
public class GenericHQLQuery<E extends Object> implements Query {

	/** HibernateQueryオブジェクト */
	private Query query;

	/**
	 * コンストラクタ<br>
	 * @param query HibernateQueryオブジェクト
	 */
	public GenericHQLQuery(Query query) {
		if (query == null) {
			throw new NullPointerException("query");
		}
		this.query = query;
	}

	/**
	 * Execute the update or delete statement.
	 * The semantics are compliant with the ejb3 Query.executeUpdate() method.
	 * @return The number of entities updated or deleted.
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#executeUpdate()
	 */
	public int executeUpdate() throws HibernateException {
		return query.executeUpdate();
	}

	/**
	 * Get lock options.
	 * @return Lock options
	 * @see org.hibernate.Query#getLockOptions()
	 */
	public LockOptions getLockOptions() {
		return query.getLockOptions();
	}

	/**
	 * Return the names of all named parameters of the query.
	 * @return the parameter names, in no particular order
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#getNamedParameters()
	 */
	public String[] getNamedParameters() throws HibernateException {
		return query.getNamedParameters();
	}

	/**
	 * Get the query string.
	 * @return the query string
	 * @see org.hibernate.Query#getQueryString()
	 */
	public String getQueryString() {
		return query.getQueryString();
	}

	/**
	 * Return the HQL select clause aliases (if any)
	 * @return an array of aliases as strings
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#getReturnAliases()
	 */
	public String[] getReturnAliases() throws HibernateException {
		return query.getReturnAliases();
	}

	/**
	 * Return the Hibernate types of the query result set.
	 * @return an array of types
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#getReturnTypes()
	 */
	public Type[] getReturnTypes() throws HibernateException {
		return query.getReturnTypes();
	}

	/**
	 * Should entities and proxies loaded by this Query be put in read-only mode? If the
	 * read-only/modifiable setting was not initialized, then the default
	 * read-only/modifiable setting for the persistence context is returned instead.
	 *      The read-only/modifiable setting has no impact on entities/proxies returned by the
	 *      query that existed in the session before the query was executed.
	 * @return true, entities and proxies loaded by the query will be put in read-only mode
	 *         false, entities and proxies loaded by the query will be put in modifiable mode
	 * @see org.hibernate.Query#isReadOnly()
	 */
	public boolean isReadOnly() {
		return query.isReadOnly();
	}

	/**
	 * Return the query results as an <tt>Iterator</tt>. If the query
	 * contains multiple results pre row, the results are returned in
	 * an instance of <tt>Object[]</tt>.<br>
	 * <br>
	 * Entities returned as results are initialized on demand. The first
	 * SQL query returns identifiers only.<br>
	 * @return the result iterator
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#iterate()
	 */
	@SuppressWarnings("unchecked")
	public Iterator<E> iterate() throws HibernateException {
		return (Iterator<E>) query.iterate();
	}

	/**
	 * Return the query results as a <tt>List</tt>. If the query contains
	 * multiple results pre row, the results are returned in an instance
	 * of <tt>Object[]</tt>.
	 * @return the result list
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#list()
	 */
	@SuppressWarnings("unchecked")
	public List<E> list() throws HibernateException {
		List<E> list = null;
		if (Config.getInteger(HibernateConfigName.HIBERNATE_RESULT_MAXCOUNT) >= 1) {
			query.setMaxResults(Config.getInteger(HibernateConfigName.HIBERNATE_RESULT_MAXCOUNT) + 1);
			list = query.list();
			if (list.size() > Config.getLong(HibernateConfigName.HIBERNATE_RESULT_MAXCOUNT)) {
				throw new HibernateTooManyResultException();
			}
		}
		return (List<E>) query.list();
	}

	/**
	 * Return the query results as <tt>ScrollableResults</tt>. The
	 * scrollability of the returned results depends upon JDBC driver
	 * support for scrollable <tt>ResultSet</tt>s.<br>
	 * @return the result iterator
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#scroll()
	 */
	public ScrollableResults scroll() throws HibernateException {
		return query.scroll();
	}

	/**
	 * Return the query results as <tt>ScrollableResults</tt>. The
	 * scrollability of the returned results depends upon JDBC driver
	 * support for scrollable <tt>ResultSet</tt>s.<br>
	 * @return the result iterator
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#scroll(org.hibernate.ScrollMode)
	 */
	public ScrollableResults scroll(ScrollMode arg0) throws HibernateException {
		return query.scroll(arg0);
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setBigDecimal(int, java.math.BigDecimal)
	 */
	public GenericHQLQuery<E> setBigDecimal(int arg0, BigDecimal arg1) {
		Query resultQuery = query.setBigDecimal(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setBigDecimal(java.lang.String, java.math.BigDecimal)
	 */
	public GenericHQLQuery<E> setBigDecimal(String arg0, BigDecimal arg1) {
		Query resultQuery = query.setBigDecimal(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setBigInteger(int, java.math.BigInteger)
	 */
	public GenericHQLQuery<E> setBigInteger(int arg0, BigInteger arg1) {
		Query resultQuery = query.setBigInteger(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setBigInteger(java.lang.String, java.math.BigInteger)
	 */
	public GenericHQLQuery<E> setBigInteger(String arg0, BigInteger arg1) {
		Query resultQuery = query.setBigInteger(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setBinary(int, byte[])
	 */
	public GenericHQLQuery<E> setBinary(int arg0, byte[] arg1) {
		Query resultQuery = query.setBinary(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setBinary(java.lang.String, byte[])
	 */
	public GenericHQLQuery<E> setBinary(String arg0, byte[] arg1) {
		Query resultQuery = query.setBinary(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setBoolean(int, boolean)
	 */
	public GenericHQLQuery<E> setBoolean(int arg0, boolean arg1) {
		Query resultQuery = query.setBoolean(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setBoolean(java.lang.String, boolean)
	 */
	public GenericHQLQuery<E> setBoolean(String arg0, boolean arg1) {
		Query resultQuery = query.setBoolean(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setByte(int, byte)
	 */
	public GenericHQLQuery<E> setByte(int arg0, byte arg1) {
		Query resultQuery = query.setByte(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setByte(java.lang.String, byte)
	 */
	public GenericHQLQuery<E> setByte(String arg0, byte arg1) {
		Query resultQuery = query.setByte(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Override the current session cache mode, just for this query.
	 * @param arg0 Cache mode
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setCacheMode(org.hibernate.CacheMode)
	 */
	public GenericHQLQuery<E> setCacheMode(CacheMode arg0) {
		Query resultQuery = query.setCacheMode(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Set the name of the cache region.
	 * @param arg0 cacheRegion the name of a query cache region, or <tt>null</tt> for the default query cache
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setCacheRegion(java.lang.String)
	 */
	public GenericHQLQuery<E> setCacheRegion(String arg0) {
		Query resultQuery = query.setCacheRegion(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Enable caching of this query result set.
	 * @param arg0 cacheable Should the query results be cacheable?
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setCacheable(boolean)
	 */
	public GenericHQLQuery<E> setCacheable(boolean arg0) {
		Query resultQuery = query.setCacheable(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setCalendar(int, java.util.Calendar)
	 */
	public GenericHQLQuery<E> setCalendar(int arg0, Calendar arg1) {
		Query resultQuery = query.setCalendar(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setCalendar(java.lang.String, java.util.Calendar)
	 */
	public GenericHQLQuery<E> setCalendar(String arg0, Calendar arg1) {
		Query resultQuery = query.setCalendar(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setCalendarDate(int, java.util.Calendar)
	 */
	public GenericHQLQuery<E> setCalendarDate(int arg0, Calendar arg1) {
		Query resultQuery = query.setCalendarDate(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setCalendarDate(java.lang.String, java.util.Calendar)
	 */
	public GenericHQLQuery<E> setCalendarDate(String arg0, Calendar arg1) {
		Query resultQuery = query.setCalendarDate(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setCharacter(int, char)
	 */
	public GenericHQLQuery<E> setCharacter(int arg0, char arg1) {
		Query resultQuery = query.setCharacter(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setCharacter(java.lang.String, char)
	 */
	public GenericHQLQuery<E> setCharacter(String arg0, char arg1) {
		Query resultQuery = query.setCharacter(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Add a comment to the generated SQL.
	 * @param arg0 comment a human-readable string
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setComment(java.lang.String)
	 */
	public GenericHQLQuery<E> setComment(String arg0) {
		Query resultQuery = query.setComment(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setDate(int, java.util.Date)
	 */
	public GenericHQLQuery<E> setDate(int arg0, Date arg1) {
		Query resultQuery = query.setDate(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setDate(java.lang.String, java.util.Date)
	 */
	public GenericHQLQuery<E> setDate(String arg0, Date arg1) {
		Query resultQuery = query.setDate(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setDouble(int, double)
	 */
	public GenericHQLQuery<E> setDouble(int arg0, double arg1) {
		Query resultQuery = query.setDouble(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setDouble(java.lang.String, double)
	 */
	public GenericHQLQuery<E> setDouble(String arg0, double arg1) {
		Query resultQuery = query.setDouble(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setEntity(int, java.lang.Object)
	 */
	public GenericHQLQuery<E> setEntity(int arg0, Object arg1) {
		Query resultQuery = query.setEntity(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setEntity(java.lang.String, java.lang.Object)
	 */
	public GenericHQLQuery<E> setEntity(String arg0, Object arg1) {
		Query resultQuery = query.setEntity(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Set a fetch size for the underlying JDBC query.
	 * @param arg0 fetchSize the fetch size
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setFetchSize(int)
	 */
	public GenericHQLQuery<E> setFetchSize(int arg0) {
		Query resultQuery = query.setFetchSize(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Set the first row to retrieve. If not set, rows will be
	 * retrieved beginnning from row <tt>0</tt>.
	 * @param arg0 firstResult a row number, numbered from <tt>0</tt>
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setFirstResult(int)
	 */
	public GenericHQLQuery<E> setFirstResult(int arg0) {
		Query resultQuery = query.setFirstResult(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setFloat(int, float)
	 */
	public GenericHQLQuery<E> setFloat(int arg0, float arg1) {
		Query resultQuery = query.setFloat(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setFloat(java.lang.String, float)
	 */
	public GenericHQLQuery<E> setFloat(String arg0, float arg1) {
		Query resultQuery = query.setFloat(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Override the current session flush mode, just for
	 * this query.
	 * @param arg0 Flush mode
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setFlushMode(org.hibernate.FlushMode)
	 */
	public GenericHQLQuery<E> setFlushMode(FlushMode arg0) {
		Query resultQuery = query.setFlushMode(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setInteger(int, int)
	 */
	public GenericHQLQuery<E> setInteger(int arg0, int arg1) {
		Query resultQuery = query.setInteger(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setInteger(java.lang.String, int)
	 */
	public GenericHQLQuery<E> setInteger(String arg0, int arg1) {
		Query resultQuery = query.setInteger(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setLocale(int, java.util.Locale)
	 */
	public GenericHQLQuery<E> setLocale(int arg0, Locale arg1) {
		Query resultQuery = query.setLocale(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setLocale(java.lang.String, java.util.Locale)
	 */
	public GenericHQLQuery<E> setLocale(String arg0, Locale arg1) {
		Query resultQuery = query.setLocale(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Set the lockmode for the objects idententified by the
	 * given alias that appears in the <tt>FROM</tt> clause.
	 * @param arg0 alias a query alias, or <tt>this</tt> for a collection filter
	 * @param arg1 lock options
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setLockMode(java.lang.String, org.hibernate.LockMode)
	 */
	public GenericHQLQuery<E> setLockMode(String arg0, LockMode arg1) {
		Query resultQuery = query.setLockMode(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Set the lock options for the objects idententified by the
	 * given alias that appears in the <tt>FROM</tt> clause.
	 * @param arg0 Lock options
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setLockOptions(org.hibernate.LockOptions)
	 */
	public GenericHQLQuery<E> setLockOptions(LockOptions arg0) {
		Query resultQuery = query.setLockOptions(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setLong(int, long)
	 */
	public GenericHQLQuery<E> setLong(int arg0, long arg1) {
		Query resultQuery = query.setLong(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setLong(java.lang.String, long)
	 */
	public GenericHQLQuery<E> setLong(String arg0, long arg1) {
		Query resultQuery = query.setLong(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Set the maximum number of rows to retrieve. If not set,
	 * there is no limit to the number of rows retrieved.
	 * @param arg0 maxResults the maximum number of rows
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setMaxResults(int)
	 */
	public GenericHQLQuery<E> setMaxResults(int arg0) {
		Query resultQuery = query.setMaxResults(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind a value to a JDBC-style query parameter.
	 * @param position the position of the parameter in the query
	 * string, numbered from <tt>0</tt>.
	 * @param arg0 index
	 * @param arg1 the possibly-null parameter value
	 * @param type the Hibernate type
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setParameter(int, java.lang.Object, org.hibernate.type.Type)
	 */
	public GenericHQLQuery<E> setParameter(int arg0, Object arg1, Type arg2) {
		Query resultQuery = query.setParameter(arg0, arg1, arg2);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#setParameter(int, java.lang.Object)
	 */
	public GenericHQLQuery<E> setParameter(int arg0, Object arg1) throws HibernateException {
		Query resultQuery = query.setParameter(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind a value to a named query parameter.
	 * @param arg0 name the name of the parameter
	 * @param arg1 the possibly-null parameter value
	 * @param arg2 type the Hibernate type
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setParameter(java.lang.String, java.lang.Object, org.hibernate.type.Type)
	 */
	public GenericHQLQuery<E> setParameter(String arg0, Object arg1, Type arg2) {
		Query resultQuery = query.setParameter(arg0, arg1, arg2);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind a value to a named query parameter. The Hibernate type of the parameter is
	 * first detected via the usage/position in the query and if not sufficient secondly 
	 * guessed from the class of the given object.
	 * @param arg0 name the name of the parameter
	 * @param arg1 the non-null parameter value
	 * @return this (for method chaining)
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#setParameter(java.lang.String, java.lang.Object)
	 */
	public GenericHQLQuery<E> setParameter(String arg0, Object arg1) throws HibernateException {
		Query resultQuery = query.setParameter(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind multiple values to a named query parameter. This is useful for binding
	 * a list of values to an expression such as <tt>foo.bar in (:value_list)</tt>.
	 * @param arg0 name the name of the parameter
	 * @param arg1 vals a collection of values to list
	 * @param arg2 type the Hibernate type of the values
	 * @return this (for method chaining)
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#setParameterList(java.lang.String, java.util.Collection, org.hibernate.type.Type)
	 */
	@SuppressWarnings("rawtypes")
	public GenericHQLQuery<E> setParameterList(String arg0, Collection arg1, Type arg2) throws HibernateException {
		Query resultQuery = query.setParameterList(arg0, arg1, arg2);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind multiple values to a named query parameter. The Hibernate type of the parameter is
	 * first detected via the usage/position in the query and if not sufficient secondly 
	 * guessed from the class of the first object in the collection. This is useful for binding a list of values
	 * to an expression such as <tt>foo.bar in (:value_list)</tt>.
	 * @param arg0 name the name of the parameter
	 * @param arg1 vals a collection of values to list
	 * @return this (for method chaining)
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#setParameterList(java.lang.String, java.util.Collection)
	 */
	@SuppressWarnings("rawtypes")
	public GenericHQLQuery<E> setParameterList(String arg0, Collection arg1) throws HibernateException {
		Query resultQuery = query.setParameterList(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind multiple values to a named query parameter. This is useful for binding
	 * a list of values to an expression such as <tt>foo.bar in (:value_list)</tt>.
	 * @param arg0 name the name of the parameter
	 * @param arg1 vals a collection of values to list
	 * @param arg2 type the Hibernate type of the values
	 * @return this (for method chaining)
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#setParameterList(java.lang.String, java.lang.Object[], org.hibernate.type.Type)
	 */
	public GenericHQLQuery<E> setParameterList(String arg0, Object[] arg1, Type arg2) throws HibernateException {
		Query resultQuery = query.setParameterList(arg0, arg1, arg2);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind multiple values to a named query parameter. The Hibernate type of the parameter is
	 * first detected via the usage/position in the query and if not sufficient secondly 
	 * guessed from the class of the first object in the array. This is useful for binding a list of values
	 * to an expression such as <tt>foo.bar in (:value_list)</tt>.
	 * @param arg0 name the name of the parameter
	 * @param arg1 vals a collection of values to list
	 * @return this (for method chaining)
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#setParameterList(java.lang.String, java.lang.Object[])
	 */
	public GenericHQLQuery<E> setParameterList(String arg0, Object[] arg1) throws HibernateException {
		Query resultQuery = query.setParameterList(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind values and types to positional parameters.
	 * @param arg0 bind values
	 * @param arg1 bind value types
	 * @return this (for method chaining)
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#setParameters(java.lang.Object[], org.hibernate.type.Type[])
	 */
	public GenericHQLQuery<E> setParameters(Object[] arg0, Type[] arg1) throws HibernateException {
		Query resultQuery = query.setParameters(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the property values of the given bean to named parameters of the query,
	 * matching property names with parameter names and mapping property types to
	 * Hibernate types using hueristics.
	 * @param arg0 bean any JavaBean or POJO
	 * @return this (for method chaining)
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#setProperties(java.util.Map)
	 */
	@SuppressWarnings("rawtypes")
	public GenericHQLQuery<E> setProperties(Map arg0) throws HibernateException {
		Query resultQuery = query.setProperties(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the property values of the given bean to named parameters of the query,
	 * matching property names with parameter names and mapping property types to
	 * Hibernate types using hueristics.
	 * @param arg0 bean any JavaBean or POJO
	 * @return this (for method chaining)
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#setProperties(java.lang.Object)
	 */
	public GenericHQLQuery<E> setProperties(Object arg0) throws HibernateException {
		Query resultQuery = query.setProperties(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Set the read-only/modifiable mode for entities and proxies
	 * loaded by this Query. This setting overrides the default setting
	 * for the persistence context.
	 * To set the default read-only/modifiable setting used for
	 * entities and proxies that are loaded into the session:
	 * Read-only entities are not dirty-checked and snapshots of persistent
	 * state are not maintained. Read-only entities can be modified, but
	 * changes are not persisted.
	 *
	 * When a proxy is initialized, the loaded entity will have the same
	 * read-only/modifiable setting as the uninitialized
	 * proxy has, regardless of the session's current setting.
	 *
	 * The read-only/modifiable setting has no impact on entities/proxies
	 * returned by the query that existed in the session before the query was executed.
	 *
	 * @param arg0 true, entities and proxies loaded by the query will be put in read-only mode
	 *                 false, entities and proxies loaded by the query will be put in modifiable mode
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setReadOnly(boolean)
	 */
	public GenericHQLQuery<E> setReadOnly(boolean arg0) {
		Query resultQuery = query.setReadOnly(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Set a strategy for handling the query results. This can be used to change
	 * "shape" of the query result.
	 * @param arg0 transformer The transformer to apply
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setResultTransformer(org.hibernate.transform.ResultTransformer)
	 */
	public GenericHQLQuery<E> setResultTransformer(ResultTransformer arg0) {
		Query resultQuery = query.setResultTransformer(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setSerializable(int, java.io.Serializable)
	 */
	public GenericHQLQuery<E> setSerializable(int arg0, Serializable arg1) {
		Query resultQuery = query.setSerializable(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setSerializable(java.lang.String, java.io.Serializable)
	 */
	public GenericHQLQuery<E> setSerializable(String arg0, Serializable arg1) {
		Query resultQuery = query.setSerializable(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setShort(int, short)
	 */
	public GenericHQLQuery<E> setShort(int arg0, short arg1) {
		Query resultQuery = query.setShort(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setShort(java.lang.String, short)
	 */
	public GenericHQLQuery<E> setShort(String arg0, short arg1) {
		Query resultQuery = query.setShort(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setString(int, java.lang.String)
	 */
	public GenericHQLQuery<E> setString(int arg0, String arg1) {
		Query resultQuery = query.setString(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setString(java.lang.String, java.lang.String)
	 */
	public GenericHQLQuery<E> setString(String arg0, String arg1) {
		Query resultQuery = query.setString(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setText(int, java.lang.String)
	 */
	public GenericHQLQuery<E> setText(int arg0, String arg1) {
		Query resultQuery = query.setText(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setText(java.lang.String, java.lang.String)
	 */
	public GenericHQLQuery<E> setText(String arg0, String arg1) {
		Query resultQuery = query.setText(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setTime(int, java.util.Date)
	 */
	public GenericHQLQuery<E> setTime(int arg0, Date arg1) {
		Query resultQuery = query.setTime(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setTime(java.lang.String, java.util.Date)
	 */
	public GenericHQLQuery<E> setTime(String arg0, Date arg1) {
		Query resultQuery = query.setTime(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Set a timeout for the underlying JDBC query.
	 * @param arg0 timeout the timeout in seconds
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setTimeout(int)
	 */
	public GenericHQLQuery<E> setTimeout(int arg0) {
		Query resultQuery = query.setTimeout(arg0);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind index
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setTimestamp(int, java.util.Date)
	 */
	public GenericHQLQuery<E> setTimestamp(int arg0, Date arg1) {
		Query resultQuery = query.setTimestamp(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Bind the values of the given Map for each named parameters of the query,
	 * matching key names with parameter names and mapping value types to
	 * Hibernate types using hueristics.
	 * @param arg0 bind key
	 * @param arg1 bind value
	 * @return this (for method chaining)
	 * @see org.hibernate.Query#setTimestamp(java.lang.String, java.util.Date)
	 */
	public GenericHQLQuery<E> setTimestamp(String arg0, Date arg1) {
		Query resultQuery = query.setTimestamp(arg0, arg1);
		if (query.equals(resultQuery)) {
			return this;
		} else {
			return new GenericHQLQuery<E>(resultQuery);
		}
	}

	/**
	 * Convenience method to return a single instance that matches
	 * the query, or null if the query returns no results.
	 * @return the single result or <tt>null</tt>
	 * @throws NonUniqueResultException if there is more than one matching result
	 * @throws HibernateException Hibernate internal exception
	 * @see org.hibernate.Query#uniqueResult()
	 */
	@SuppressWarnings("unchecked")
	public E uniqueResult() throws HibernateException {
		return ((E) query.uniqueResult());
	}
}
