package org.phosphoresce.webcore.ext.hibernate4.transaction;

import org.hibernate.ObjectNotFoundException;
import org.hibernate.PessimisticLockException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.StaleObjectStateException;
import org.hibernate.Transaction;
import org.hibernate.dialect.lock.LockingStrategyException;
import org.hibernate.exception.ConstraintViolationException;
import org.phosphoresce.lib.commons.util.StringUtil;
import org.phosphoresce.webcore.core.transaction.TransactionManager;
import org.phosphoresce.webcore.ext.hibernate4.HibernateConstants;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateEntityDuplicateException;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateEntityLockException;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateEntityModificationException;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateEntityNoExistException;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateTransactionException;

/**
 * Hibernate(4.X系)を利用したデータベースアクセスロジックのトランザクション管理機能を提供します。<br>
 * <p>
 * このクラスはコアモジュールの{@link TransactionManager}で標準化されているインタフェースが実装されています。<br>
 * また、コアモジュールのトランザクションマネージャクラスで定義されているコンストラクタ設計基準に準拠したコンストラクタが提供されます。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 * 2013/08/27	Kitagawa		セッションファクトリクラスはシングルトンで使い回し、トランザクション毎にクローズしないように修正
 *-->
 */
public final class HibernateTransactionManager extends TransactionManager implements HibernateConstants {

	/** セッションファクトリオブジェクト */
	private SessionFactory factory;

	/** セッションオブジェクト */
	private Session session;

	/** トランザクションオブジェクト */
	private Transaction transaction;

	/**
	 * コンストラクタ<br>
	 */
	public HibernateTransactionManager() {
		super();
	}

	/**
	 * コンストラクタ<br>
	 * @param namespace 名前空間
	 */
	public HibernateTransactionManager(String namespace) {
		super(namespace);
	}

	/**
	 * Hibernateセッションオブジェクトを取得します。<br>
	 * @return session Hibernateセッションオブジェクト
	 */
	Session getSession() {
		if (!isOpened()) {
			throw new HibernateTransactionException("FHBN00013");
		}
		return session;
	}

	/**
	 * トランザクション開始処理を実行します。<br>
	 * @see org.phosphoresce.webcore.core.transaction.TransactionManager#openProcess()
	 */
	@Override
	protected void openProcess() {
		try {
			if (factory == null || factory.isClosed()) {
				if (!StringUtil.isEmpty(getNamespace())) {
					factory = TransactionFactory.instance().createSessionFactory(getNamespace());
				} else {
					factory = TransactionFactory.instance().createSessionFactory();
				}
			}
			session = factory.openSession();
			transaction = session.beginTransaction();
		} catch (Throwable e) {
			throw new HibernateTransactionException("FHBN00015", e);
		}
	}

	/**
	 * トランザクションを終了処理を実行します。<br>
	 * @see org.phosphoresce.webcore.core.transaction.TransactionManager#closeProcess()
	 */
	@Override
	protected void closeProcess() {
		try {
			if (!(transaction.wasCommitted() || transaction.wasRolledBack())) {
				transaction.rollback();
			}
			session.close();
			session = null;
			transaction = null;
			// ファクトリクローズは初期化処理クラスで実施する
			//factory.close();
			//factory = null;
		} catch (Throwable e) {
			throw new HibernateTransactionException("FHBN00016", e);
		}
	}

	/**
	 * トランザクションをコミット処理を実行します。<br>
	 * @see org.phosphoresce.webcore.core.transaction.TransactionManager#commitProcess()
	 */
	@Override
	protected void commitProcess() {
		try {
			if (transaction.isActive()) {
				transaction.commit();
			}
		} catch (LockingStrategyException e) {
			log.output(e, "FHBN00033");
			throw new HibernateEntityLockException(e);
		} catch (PessimisticLockException e) {
			log.output(e, "FHBN00033");
			throw new HibernateEntityLockException(e);
		} catch (StaleObjectStateException e) {
			log.output(e, "FHBN00033");
			throw new HibernateEntityModificationException(e);
		} catch (ObjectNotFoundException e) {
			log.output(e, "FHBN00033");
			throw new HibernateEntityNoExistException(e);
		} catch (ConstraintViolationException e) {
			log.output(e, "FHBN00033");
			throw new HibernateEntityDuplicateException(e);
		}
	}

	/**
	 * トランザクションをロールバック処理を実行します。<br>
	 * @see org.phosphoresce.webcore.core.transaction.TransactionManager#rollbackProcess()
	 */
	@Override
	protected void rollbackProcess() {
		if (transaction.isActive()) {
			transaction.rollback();
		}
	}
}
