package org.phosphoresce.webcore.ext.hibernate4.transaction;

import java.util.HashMap;
import java.util.Map;

import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.service.ServiceRegistryBuilder;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateTransactionException;

/**
 * Hibernateトランザクションの{@link SessionFactory}クラスインスタンス生成を行うインタフェースを提供します。<br>
 * <p>
 * 当クラスはpublicなクラススコープとして提供されますが、許可されるインタフェースは初期化処理のみとなります。<br>
 * {@link SessionFactory}クラスインスタンス生成はすべてアドオンモジュール内のアクセスに制限され、隠蔽化するとともに標準化されています。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/07	Kitagawa		新規作成
 * 2013/08/27	Kitagawa		セッションファクトリをシングルトンで使い回すように修正
 *-->
 */
public final class TransactionFactory {

	/** クラスインスタンス */
	private static TransactionFactory instance;

	/** Hibernate環境設定オブジェクトキャッシュ */
	private Map<String, Configuration> configurations;

	/** ディフォルトHibernate環境設定オブジェクトキャッシュ */
	private Configuration defaultConfiguration;

	/** Hibernateセッションファクトリオブジェクト */
	private Map<String, SessionFactory> factories;

	/** ディフォルトHibernateセッションファクトリオブジェクト */
	private SessionFactory defaultFactory;

	/**
	 * コンストラクタ<br>
	 */
	private TransactionFactory() {
		super();
		this.configurations = new HashMap<String, Configuration>();
		this.defaultConfiguration = null;
		this.factories = new HashMap<String, SessionFactory>();
		this.defaultFactory = null;
	}

	/**
	 * クラスインスタンスを取得します。<br>
	 * @return クラスインスタンス
	 */
	static TransactionFactory instance() {
		if (instance == null) {
			throw new HibernateTransactionException("FHBN00017");
		}
		return instance;
	}

	/**
	 * 指定された名前空間で環境設定オブジェクトを設定します。<br>
	 * @param namespace 名前空間
	 * @param configuration 初期化済み環境設定オブジェクト
	 */
	public synchronized static void initialize(String namespace, Configuration configuration) {
		if (instance == null) {
			instance = new TransactionFactory();
		}
		synchronized (instance) {
			if (instance.configurations.containsKey(namespace)) {
				throw new HibernateTransactionException("FHBN00019", new String[] { namespace });
			}
			instance.configurations.put(namespace, configuration);

			// Hibernateセッションテスト生成処理(セッションファクトリインスタンス生成、スキーマエクスポート)
			//instance.createSessionFactory(namespace).close();
			instance.createSessionFactory(namespace);
		}
	}

	/**
	 * ディフォルトの環境設定オブジェクトを設定します。<br>
	 * @param configuration 初期化済み環境設定オブジェクト
	 */
	public synchronized static void initialize(Configuration configuration) {
		if (instance == null) {
			instance = new TransactionFactory();
		}
		synchronized (instance) {
			if (instance.defaultConfiguration != null) {
				throw new HibernateTransactionException("FHBN00020");
			}
			instance.defaultConfiguration = configuration;

			// Hibernateセッションテスト生成処理(セッションファクトリインスタンス生成、スキーマエクスポート)
			//instance.createSessionFactory().close();
			instance.createSessionFactory();
		}
	}

	/**
	 * インスタンスを破棄します。<br>
	 */
	public synchronized static void destroy() {
		synchronized (instance) {
			synchronized (instance.defaultFactory) {
				instance.defaultFactory.close();
				instance.defaultFactory = null;
			}
			synchronized (instance.factories) {
				for (SessionFactory factory : instance.factories.values()) {
					factory.close();
				}
				instance.factories.clear();
			}
			synchronized (instance.defaultConfiguration) {
				instance.defaultConfiguration = null;
			}
			synchronized (instance.configurations) {
				instance.configurations.clear();
			}
			instance = null;
		}
	}

	/**
	 * セッションファクトリクラスインスタンスを生成します。<br>
	 * @return セッションファクトリクラスインスタンス
	 */
	SessionFactory createSessionFactory() {
		if (defaultFactory == null || defaultFactory.isClosed()) {
			if (defaultConfiguration == null) {
				throw new HibernateTransactionException("FHBN00018");
			}
			ServiceRegistry registry = new ServiceRegistryBuilder().applySettings(defaultConfiguration.getProperties()).buildServiceRegistry();
			defaultFactory = defaultConfiguration.buildSessionFactory(registry);
		}
		return defaultFactory;
	}

	/**
	 * セッションファクトリクラスインスタンスを生成します。<br>
	 * @param namespace 名前空間
	 * @return セッションファクトリクラスインスタンス
	 */
	SessionFactory createSessionFactory(String namespace) {
		SessionFactory factory = factories.get(namespace);
		if (factory == null || factory.isClosed()) {
			if (!configurations.containsKey(namespace)) {
				throw new HibernateTransactionException("FHBN00021", new String[] { namespace });
			}
			ServiceRegistry registry = new ServiceRegistryBuilder().applySettings(configurations.get(namespace).getProperties()).buildServiceRegistry();
			factories.put(namespace, configurations.get(namespace).buildSessionFactory(registry));
		}
		return factories.get(namespace);
	}
}
